#pragma once

#include <eopcc/cloud.hpp>
#include <eopcc/grid_structs.hpp>

namespace eopcc {

// Defines the registration search space and refine factor
struct RegSearchSpace {
  Point center;
  double start;
  double stop;
  double delta;

  double refine_factor;
  double stoping_factor;

  RegSearchSpace() : center(0,0,0) {
    start = -20;
    stop = 20;
    delta = 5;
    refine_factor = 0.5;
    stoping_factor = 0.5;
  }

  RegSearchSpace(double mag, double nsteps) : center(0,0,0) {
    start = -mag;
    stop = mag;
    delta = mag/nsteps;
      refine_factor = 0.5;
    stoping_factor = 0.5;
  }

  RegSearchSpace refine(double factor, Point ncenter) const  {
    RegSearchSpace nreg;
    nreg.start = start * factor;
    nreg.stop = stop * factor;
    nreg.delta = delta * factor;
    nreg.center = ncenter;
    nreg.refine_factor = refine_factor;
    nreg.stoping_factor = stoping_factor;

    return nreg;
  }
};

/// Registers and returns the shift from the translation with minimum RMS error against the truth_grid
Point RegisterFromRmsGrid(const RegSearchSpace& regspace, const PointCloud& input, const eopcc::GridModel<double>& truth_grid);

/// Registers and returns the shift from the translation with minimum Median error against the truth_grid
Point RegisterFromMedianGrid(const RegSearchSpace& regspace, const PointCloud& input, const eopcc::GridModel<double>& truth_grid);

}
